@@ -10,7 +10,7 @@ import sys
 from autouri import GCSURI, AutoURI
 
 from . import __version__ as version
-from .caper_args import get_parser_and_defaults
+from .caper_args import ResourceAnalysisReductionMethod, get_parser_and_defaults
 from .caper_client import CaperClient, CaperClientSubmit
 from .caper_init import init_caper_conf
 from .caper_labels import CaperLabels
@@ -22,15 +22,20 @@ from .cromwell_backend import (
 )
 from .cromwell_metadata import CromwellMetadata
 from .dict_tool import flatten_dict
+from .resource_analysis import LinearResourceAnalysis
 from .server_heartbeat import ServerHeartbeat
 
 logger = logging.getLogger(__name__)
 
 
-DEFAULT_TMP_DIR_NAME = '.caper_tmp'
 DEFAULT_DB_FILE_PREFIX = 'caper_file_db'
 DEFAULT_SERVER_HEARTBEAT_FILE = '~/.caper/default_server_heartbeat'
-USER_INTERRUPT_WARNING = '\n********** DO NOT CTRL+C MULTIPLE TIMES **********\n'
+USER_INTERRUPT_WARNING = (
+    '\n\n'
+    '*** DO NOT CTRL+C MULTIPLE TIMES! ***\n'
+    '*** OR CAPER WILL NOT BE ABLE TO STOP CROMWELL AND RUNNING WORKFLOWS/TASKS ***'
+    '\n\n'
+)
 REGEX_DELIMITER_GCP_PARAMS = r',| '
 PRINT_ROW_DELIMITER = '\t'
 
@@ -115,20 +120,28 @@ def check_dirs(args):
     if hasattr(args, 'local_out_dir'):
         args.local_out_dir = get_abspath(args.local_out_dir)
         if not args.local_loc_dir:
-            args.local_loc_dir = os.path.join(args.local_out_dir, DEFAULT_TMP_DIR_NAME)
+            args.local_loc_dir = os.path.join(
+                args.local_out_dir, CaperRunner.DEFAULT_LOC_DIR_NAME
+            )
     else:
         if not args.local_loc_dir:
-            args.local_loc_dir = os.path.join(os.getcwd(), DEFAULT_TMP_DIR_NAME)
+            args.local_loc_dir = os.path.join(
+                os.getcwd(), CaperRunner.DEFAULT_LOC_DIR_NAME
+            )
 
     args.local_loc_dir = get_abspath(args.local_loc_dir)
 
     if hasattr(args, 'gcp_out_dir'):
         if args.gcp_out_dir and not args.gcp_loc_dir:
-            args.gcp_loc_dir = os.path.join(args.gcp_out_dir, DEFAULT_TMP_DIR_NAME)
+            args.gcp_loc_dir = os.path.join(
+                args.gcp_out_dir, CaperRunner.DEFAULT_LOC_DIR_NAME
+            )
 
     if hasattr(args, 'aws_out_dir'):
         if args.aws_out_dir and not args.aws_loc_dir:
-            args.aws_loc_dir = os.path.join(args.aws_out_dir, DEFAULT_TMP_DIR_NAME)
+            args.aws_loc_dir = os.path.join(
+                args.aws_out_dir, CaperRunner.DEFAULT_LOC_DIR_NAME
+            )
 
 
 def check_db_path(args):
@@ -190,6 +203,7 @@ def runner(args, nonblocking_server=False):
         postgresql_db_name=args.postgresql_db_name,
         gcp_prj=args.gcp_prj,
         use_google_cloud_life_sciences=args.use_google_cloud_life_sciences,
+        gcp_region=args.gcp_region,
         gcp_zones=args.gcp_zones,
         gcp_call_caching_dup_strat=args.gcp_call_caching_dup_strat,
         gcp_out_dir=args.gcp_out_dir,
@@ -273,6 +287,8 @@ def client(args):
             subcmd_troubleshoot(c, args)
         elif args.action == 'gcp_monitor':
             subcmd_gcp_monitor(c, args)
+        elif args.action == 'gcp_res_analysis':
+            subcmd_gcp_res_analysis(c, args)
         elif args.action == 'cleanup':
             subcmd_cleanup(c, args)
         else:
@@ -295,30 +311,33 @@ def subcmd_server(caper_runner, args, nonblocking=False):
             heartbeat_timeout=args.server_heartbeat_timeout,
         )
 
+    args_from_cli = {
+        'default_backend': args.backend,
+        'server_port': args.port,
+        'server_heartbeat': sh,
+        'custom_backend_conf': get_abspath(args.backend_file),
+        'embed_subworkflow': True,
+        'java_heap_server': args.java_heap_server,
+        'dry_run': args.dry_run,
+    }
+
+    if nonblocking:
+        return caper_runner.server(fileobj_stdout=sys.stdout, **args_from_cli)
+
     cromwell_stdout = get_abspath(args.cromwell_stdout)
     with open(cromwell_stdout, 'w') as f:
         try:
-            thread = caper_runner.server(
-                default_backend=args.backend,
-                server_port=args.port,
-                server_heartbeat=sh,
-                custom_backend_conf=get_abspath(args.backend_file),
-                fileobj_stdout=sys.stdout if nonblocking else f,
-                embed_subworkflow=True,
-                java_heap_server=args.java_heap_server,
-                dry_run=args.dry_run,
-            )
-            if nonblocking:
-                return thread
+            thread = caper_runner.server(fileobj_stdout=f, **args_from_cli)
             if thread:
                 thread.join()
                 if thread.returncode:
                     logger.error(
                         'Check stdout/stderr in {file}'.format(file=cromwell_stdout)
                     )
+        except KeyboardInterrupt:
+            logger.error(USER_INTERRUPT_WARNING, exc_info=True)
 
-        except Exception:
-            logger.error(USER_INTERRUPT_WARNING)
+        finally:
             if thread:
                 thread.stop()
 
@@ -359,8 +378,10 @@ def subcmd_run(caper_runner, args):
                         'Check stdout/stderr in {file}'.format(file=cromwell_stdout)
                     )
 
-        except Exception:
-            logger.error(USER_INTERRUPT_WARNING)
+        except KeyboardInterrupt:
+            logger.error(USER_INTERRUPT_WARNING, exc_info=True)
+
+        finally:
             if thread:
                 thread.stop()
 
@@ -476,18 +497,58 @@ def get_single_cromwell_metadata_obj(caper_client, args, subcmd):
     if metadata_file.exists:
         metadata = json.loads(metadata_file.read())
     else:
-        m = caper_client.metadata(
+        metadata_objs = caper_client.metadata(
             wf_ids_or_labels=args.wf_id_or_label, embed_subworkflow=True
         )
-        if len(m) > 1:
+        if len(metadata_objs) > 1:
             raise ValueError('Found multiple workflows matching with search query.')
-        elif len(m) == 0:
+        elif len(metadata_objs) == 0:
             raise ValueError('Found no workflow matching with search query.')
-        metadata = m[0]
+        metadata = metadata_objs[0]
 
     return CromwellMetadata(metadata)
 
 
+def split_list_into_file_and_non_file(lst):
+    """Returns tuple of (list of existing files, list of non-file strings)
+    """
+    files = []
+    non_files = []
+
+    for maybe_file in lst:
+        if AutoURI(get_abspath(maybe_file)).exists:
+            files.append(maybe_file)
+        else:
+            non_files.append(maybe_file)
+
+    return files, non_files
+
+
+def get_multi_cromwell_metadata_objs(caper_client, args):
+    if not args.wf_id_or_label:
+        raise ValueError(
+            'Define at least one metadata JSON file or '
+            'a search query for workflow ID/string label '
+            'if there is a running Caper server.'
+        )
+
+    files, non_files = split_list_into_file_and_non_file(args.wf_id_or_label)
+
+    all_metadata = []
+    for file in files:
+        metadata = json.loads(AutoURI(get_abspath(file)).read())
+        all_metadata.append(metadata)
+
+    if non_files:
+        all_metadata.extend(
+            caper_client.metadata(wf_ids_or_labels=non_files, embed_subworkflow=True)
+        )
+
+    if not all_metadata:
+        raise ValueError('Found no metadata/workflow matching with search query.')
+    return [CromwellMetadata(m) for m in all_metadata]
+
+
 def subcmd_troubleshoot(caper_client, args):
     cm = get_single_cromwell_metadata_obj(caper_client, args, 'troubleshoot/debug')
     cm.troubleshoot(
@@ -509,28 +570,26 @@ def subcmd_gcp_monitor(caper_client, args):
         See description at CromwellMetadata.gcp_monitor.__doc__.
         Use a JSON format instead to get more detailed information.
     """
-    cm = get_single_cromwell_metadata_obj(caper_client, args, 'gcp_profile')
-    workflow_data = cm.gcp_monitor()
+    all_metadata = get_multi_cromwell_metadata_objs(caper_client, args)
+    writer = csv.writer(sys.stdout, delimiter=PRINT_ROW_DELIMITER)
 
-    if workflow_data:
-        if args.json_format:
-            print(json.dumps(workflow_data, indent=4))
-            return
+    result = []
+    for metadata in all_metadata:
+        result.extend(metadata.gcp_monitor())
 
-        writer = csv.writer(sys.stdout, delimiter=PRINT_ROW_DELIMITER)
-
-        first_data = copy.deepcopy(workflow_data[0])
+    if args.json_format:
+        print(json.dumps(result, indent=4))
+    else:
+        # input_file_sizes is dynamic in length so exclude and then put it back
+        first_data = copy.deepcopy(result[0])
         first_data.pop('input_file_sizes')
-
-        # extract header from first data
-        flattened_key_tuples = flatten_dict(first_data).keys()
-        header = list(['.'.join(tup) for tup in flattened_key_tuples])
-        header += ['input_file_name_size_pairs']
+        header = list(flatten_dict(first_data, reducer='.').keys())
+        header += ['input_file_var_size_pairs']
         writer.writerow(header)
 
-        for task_data in workflow_data:
+        for task_data in result:
             input_file_sizes = task_data.pop('input_file_sizes')
-            row = [str(v) for _, v in flatten_dict(task_data).items()]
+            row = [str(v) for v in flatten_dict(task_data).values()]
 
             # append `input_file_sizes` data which couldn't be cleanly
             # flattened by flatten_dict()
@@ -545,6 +604,36 @@ def subcmd_gcp_monitor(caper_client, args):
             writer.writerow(row)
 
 
+def read_json(json_file):
+    if json_file:
+        json_contents = AutoURI(get_abspath(json_file)).read()
+        return json.loads(json_contents)
+
+
+def subcmd_gcp_res_analysis(caper_client, args):
+    """Solves linear regression problem to find coeffs and intercept
+    to help optimizing resources for a task based on task's input file size.
+
+    Prints out found coeffs and intercept along with raw dataset (x, y).
+        - x: input file sizes for a task
+        - y: resources (max_mem, max_disk) taken for a task
+    """
+    all_metadata = get_multi_cromwell_metadata_objs(caper_client, args)
+
+    res_analysis = LinearResourceAnalysis()
+    res_analysis.collect_resource_data(all_metadata)
+
+    result = res_analysis.analyze(
+        in_file_vars=read_json(args.in_file_vars_def_json),
+        reduce_in_file_vars=getattr(
+            ResourceAnalysisReductionMethod, args.reduce_in_file_vars
+        ).value,
+        target_resources=args.target_resources,
+        plot_pdf=get_abspath(args.plot_pdf),
+    )
+    print(json.dumps(result, indent=4))
+
+
 def subcmd_cleanup(caper_client, args):
     """Cleanup outputs of a workflow.
     """
